using namespace std;

class Matrix; // forward declaration

class SortList {

 public:
  
  // Class data (sorting lists)
  double L,H,Volume,dx,dy; // domain dimensions
  int ncell, ncellx, ncelly; // number of cells in x-dir, y-dir, total
  Matrix cellVertex; // coordinates of lower left corner for each cell
  vector<double> cell_n, index, Xref; // indexing arrays

  // Regular Constructor. 
 SortList(double L_in,double H_in,double Volume_in,double dx_in,double dy_in,int ncell_in,int ncellx_in,int ncelly_in,int npart_in):
    L(L_in),
    H(H_in),
    Volume(Volume_in),
    dx(dx_in),
    dy(dy_in),
    ncell(ncell_in),
    ncellx(ncellx_in),
    ncelly(ncelly_in),
    cellVertex(ncell_in,2),
    cell_n(ncell_in+1),  // Allocate memory (add an extra element so that 0th element can be ignored and we can count from 1)
    index(ncell_in+1),
    Xref(npart_in+1)
      {
	// initialize lists
	initLists(ncell_in,npart_in);
	
	// build cellVertex matrix
	for (int i=1;i<=ncelly;i++) 
	  for (int j=1;j<=ncellx;j++) 
	    {
	      int c=ncellx*(i-1)+j;
	      cellVertex(c,1)=(j-1)*dx;
	      cellVertex(c,2)=(i-1)*dy;
	    }
      }
  
  // Destructor. Called when a SortList object goes out of scope.
  ~SortList() {}

  // Initialization routine (re-dimensions the arrays if necessary)
  void initLists(int ncell,int npart) {   
    cell_n.assign(ncell+1,0);
    index.assign(ncell+1,0);
    Xref.assign(npart+1,0);
  }
  
  void vect_display(const vector<double> &v)
  {
    cout << "[";
    std::copy(v.begin()+1, v.end(),
	      std::ostream_iterator<int>(std::cout, " "));
    cout << "]";
  }

  void display()
  {
    cout << "\n--- display::SortList (start)";
    //printf("\n[L,H,Volume,dx,dy]=[%e,%e,%e,%e,%e]",L,H,Volume,dx,dy);
    //printf("\n[ncell,ncellx,ncelly]=[%i,%i,%i]",ncell,ncellx,ncelly);
    cout << "\ncell_n="; vect_display(cell_n);
    cout << "\nindex="; vect_display(index);
    cout << "\nXref="; vect_display(Xref);
    cout << "\ncellVertex="; cellVertex.display();
    cout << "\n--- display::SortList (end)";
  }
  
  //*********************************************************
  
 private:
  
}; // Class SortList

