#include "Dsmc.h"
#include "SortListSTL.h"

double rand( long& seed );

using namespace Parameters;

void sorter( Matrix& x, Matrix& v, int& npart, double L, double H, SortList &sD ) {
// sorter - Function to sort particles into cells
// Inputs
//    x       Positions of particles
//    L       System size
//    sD      Object containing lists used in sorting
// Output
//    sD      Object containing lists used in sorting

  //* Find the cell address for each particle
  int 
    ncellx = sD.ncellx, // ncol
    ncelly = sD.ncelly, // nrow
    ncell = sD.ncell;
  int jcell, ipart, *jx;
  jx = new int [npart+1];
  for( ipart=1; ipart<=npart; ipart++ ) {
    jx[ipart] = particlePosition2CellAddress(ipart,x,L,H,ncellx,ncelly);
    //if(Debug) printf("\nsorter::[ ipart x(i,1) x(i,2) jx[ipart] ]=[ %i %f %f %i ]",ipart, x(ipart,1), x(ipart,2), int(jx[ipart]));
  }

  sD.initLists(ncell,npart); // re-initialize and re-size the arrays
  //* Count the number of particles in each cell -> cell_n
  for( ipart=1; ipart<=npart; ipart++ )
    {
      sD.cell_n[ jx[ipart] ]++;
      //if(Debug) printf("\nsorter::[ ipart cell_n[jx[ipart]] ]=[ %i %i ]",ipart, int(sD.cell_n[jx[ipart]]));
    }

  //* Build index list as cumulative sum of the 
  //  number of particles in each cell -> index
  int m=1;
  for( jcell=1; jcell<=ncell; jcell++ ) {
    sD.index[jcell] = m;
    m += sD.cell_n[jcell];
  }

  //* Build cross-reference list -> Xref
  int *temp;
  temp = new int [ncell+1];	  // Temporary array
  for( jcell=1; jcell<=ncell; jcell++ )
    temp[jcell] = 0;
  for( ipart=1; ipart<=npart; ipart++ )	{
    jcell = jx[ipart];       // Cell address of ipart
    int k = sD.index[jcell] + temp[jcell];
    sD.Xref[k] = ipart;
    temp[jcell] = temp[jcell] + 1;
  }

  if(Debug)
    {
      sD.display();
      x.display("\nsorter::x=");
      v.display("\nsorter::v=");
    }

  delete [] jx;
  delete [] temp;

}

