#ifndef TRACKER_H_
#define TRACKER_H_

//=========================================================================================
//  Define symbolic constants and macros
//=========================================================================================
#define PI 3.1415926535897932384626433832795 
#define AVOG 6.02214179e23 // Avogadro's number
#define PROTON 1.602176487e-19 // elementary positive charge [C]
#define KB 1.3806504e-23 // Boltzmann constant [J/K]

#define ForBoundary(side,axis)  for( axis=0; axis<mg.numberOfDimensions(); axis++ ) \
    for( side=0; side<=1; side++ )
#define ForParticlesAndDims(particle,dim) for (particle=0; particle<NoOfParticles; particle++) \
    for (dim=0; dim<Dim; dim++)
#define ForSims(sim) for (sim=1; sim<=NoOfSims; sim++)
#define ForSimsMPI(sim) for (sim=rank+1; sim<=NoOfSims; sim+=size)
#define ForSteps(step) for (step=0; step<NoOfTimeSteps; step++)
#define ForProdSteps(prodstep) for (prodstep=0; prodstep<NoOfProdSteps; prodstep++)
#define ForParticles(particle) for (particle=0; particle<NoOfParticles; particle++)
#define ForParticlesReverse(particle) for (particle=NoOfParticles-1; particle>=0; particle--)
#define ForParticles2(particle2) for (particle2=0; particle2<NoOfParticles; particle2++)
#define ForDims(dim)  for (dim=0; dim<Dim; dim++)
#define ForDims2(dim,DIM)  for (dim=0; dim<DIM; dim++)
#define ForGrids(grid) for( grid=0; grid<cg.numberOfComponentGrids(); grid++ ) // loop over component grids 
#define For3I(i1,i2,i3,I1,I2,I3) for( i3=I3.getBase(); i3<=I3.getBound(); i3++ )  for( i2=I2.getBase(); i2<=I2.getBound(); i2++ )  for( i1=I1.getBase(); i1<=I1.getBound(); i1++ )
#define For3I_Interior(i1,i2,i3,I1,I2,I3) for( i3=I3.getBase()+1; i3<I3.getBound(); i3++ )  for( i2=I2.getBase()+1; i2<I2.getBound(); i2++ )  for( i1=I1.getBase()+1; i1<I1.getBound(); i1++ )
#define ForSourcePointsInChannel_old(i1,i2,i3,I1,I2,I3) for( i3=I3.getBase()+1; i3<(I3.getBound()+1)/2; i3++ )  for( i2=I2.getBase()+1; i2<(I2.getBound()+1)/2; i2++ )  for( i1=I1.getBound()/2; i1<=I1.getBound()/2; i1++ ) // source points located at interior of a quarter of the left-most interior cross-section
#define ForSourcePointsInChannel(i1,i1start,i1stop,i2,i2start,i2stop,i2step,i3,i3start,i3stop,i3step) for( i3=i3start; i3<=i3stop; i3=i3+i3step )  for( i2=i2start; i2<=i2stop; i2=i2+i2step )  for( i1=i1start; i1<=i1stop; i1++ )
#define ForAxis(axis)  for( axis=0; axis<mg.numberOfDimensions(); axis++ )
#define ForDyadicComponents(row,col)  for( int row=0; row<Dim; row++)	\
    for( int col=0; col<Dim; col++ )
#define boundary BCTypes::boundary1+side+2*axis

#define MSRGYR(step)  PPD(row_msrgyr,step)
#define MSEED(step)   PPD(row_mseed,step)
#define STRETCH(step) PPD(row_stretch,step)
#define MSD(step)     PPD(row_msd,step)
#define VAF(step)     PPD(row_vaf,step)
#define COM(dim,step) PPD(row_com+dim,step)
#define VCOM_AVG(dim) PPS(row_vcom_avg+dim)
#define DIFF_MSD      PPS(row_diff_msd)
#define DIFF_VAF      PPS(row_diff_vaf)
#define TRANSLOC(sim) PPSIM(row_transloc,sim)

#define RG00(step)  PPD(row_rg00,step)
#define RG01(step)  PPD(row_rg01,step)
#define RG02(step)  PPD(row_rg02,step)
#define RG10(step)  PPD(row_rg10,step)
#define RG11(step)  PPD(row_rg11,step)
#define RG12(step)  PPD(row_rg12,step)
#define RG20(step)  PPD(row_rg20,step)
#define RG21(step)  PPD(row_rg21,step)
#define RG22(step)  PPD(row_rg22,step)

//=========================================================================================
//  Global variables
//=========================================================================================
enum labels {inflow2=1, inflow1, outflow1, outflow2, slipWall, noSlipWall, noSlipWall2, noSlipWall3, periodic}; // label boundary conditions (start from 1, since we test if bc > 0)
enum indicesPPD {i_msrgyr, i_mseed, i_stretch, i_msd, i_vaf, i_com, i_rg00, i_rg01, i_rg02, i_rg10, i_rg11, i_rg12, i_rg20, i_rg21, i_rg22}; // indices for dynamic polymer properties matrix
enum indicesPPS {i_diff_msd, i_diff_vaf, i_vcom_avg}; // indices for static polymer properties matrix
enum indicesPPSIM {i_transloc};
const int ElStokeslet_Form=2;

CompositeGrid cg;
struct BC // declare structure for boundary conditions (Navier-Stokes)
{
  double p_inflow1; 
  double p_inflow2; 
  double p_outflow1; 
  double p_outflow2; 
  double v_wall;
} BC_NS = {0.,0.,0.,0.,0.};

struct fe // forceEval() data
{
  double 
    f_MS,
    R,
    E,
    f_H,
    v_EV, 
    Nks, 
    Rs2, 
    vdw_e, 
    vdw_s, 
    exp_EV, 
    f_EV1, 
    f_EV2, 
    lb, 
    f_ES,
    k_B,
    r0,
    k_A,
    theta0;
};

//=========================================================================================
//  Function prototypes
//=========================================================================================
float ranf(); //prototype random generator routine
float box_muller(float, float); // prototype Box Muller transform routine
int Partition(int low,int high,double arr[]); // prototype QuickSort routines
void Quick_sort(int low,int high,double arr[]);
string StringToLower(string); // prototype string conversion to lowercase routine
double find_minmax(double arr[], int size, int minmax);
int NaN(realCompositeGridFunction & u);
// Polymer processing
void ppInit (doubleArray & PPD, doubleArray & PPS, doubleArray & PPSIM, doubleArray & x0, doubleArray & v0, doubleArray & norm);
void ppUpdate(doubleArray & PPD, doubleArray & PPS, doubleArray & PPSIM, const int prodstep, const doubleArray & position, const doubleArray & position_aff, const doubleArray & velocity, const doubleArray & velocity_aff, doubleArray & x0, doubleArray & v0, doubleArray & N0, doubleArray & norm, int & TransStatus, realArray & domain_min, realArray & domain_max, doubleArray & com, doubleArray & com_aff, doubleArray & vcom, doubleArray & eev);
void ppAverage(doubleArray & PPD, doubleArray & PPS, const double dt_nondim, const doubleArray & norm);
void ppExport(doubleArray & PPD, doubleArray & PPS, doubleArray & PPSIM, const doubleArray & time, const double dt_nondim, aString nameOfHDFFile, double std_diff, const double & diff_rouse, const double & diff_zimm);

// False-Transients: general
int applyBoundaryConditions( realCompositeGridFunction & u, Real t);
int timeSteppingError(const realCompositeGridFunction & u, 
		      realCompositeGridFunction & uold,
		      const double epsilon);
// False-Transients: Navier-Stokes
int applyBoundaryConditionsNS( realCompositeGridFunction & u, Real t);
int dudt(realCompositeGridFunction & u, 
	 realCompositeGridFunction & ut, 
	 realCompositeGridFunction & f, 
	 Real t);
int euler(Real & t, 
	  Real dt,
	  realCompositeGridFunction & f,
	  realCompositeGridFunction & u1, 
	  realCompositeGridFunction & u2);
int rungeKutta4(Real & t, 
		Real dt,
		realCompositeGridFunction & f,
		realCompositeGridFunction & u1, 
		realCompositeGridFunction & u2, 
		realCompositeGridFunction & u3, 
		realCompositeGridFunction & u4);
int adamsBashforth(Real & t, 
		   Real dt,
		   realCompositeGridFunction & f,
		   realCompositeGridFunction & u, 
		   realCompositeGridFunction & u0,
		   realCompositeGridFunction & u1,
		   realCompositeGridFunction & u2,
		   realCompositeGridFunction & u3);
int 
linearMultiStep(Real & t, 
		Real dt,
		realCompositeGridFunction & f,
		realCompositeGridFunction & u,  
		realCompositeGridFunction & u0, 
		realCompositeGridFunction & u1, 
		realCompositeGridFunction & u2);
// False-Transients: Poisson-Boltzmann (no 'f')
int applyBoundaryConditionsPB( realCompositeGridFunction & u, Real t);
int dudt(realCompositeGridFunction & u, 
	 realCompositeGridFunction & ut, 
	 Real t);
int euler(Real & t, 
	  Real dt,
	  realCompositeGridFunction & u1, 
	  realCompositeGridFunction & u2);
int rungeKutta4(Real & t, 
		Real dt,
		realCompositeGridFunction & u1, 
		realCompositeGridFunction & u2, 
		realCompositeGridFunction & u3, 
		realCompositeGridFunction & u4);
int adamsBashforth(Real & t, 
		   Real dt,
		   realCompositeGridFunction & u, 
		   realCompositeGridFunction & u0,
		   realCompositeGridFunction & u1,
		   realCompositeGridFunction & u2,
		   realCompositeGridFunction & u3);
int 
linearMultiStep(Real & t, 
		Real dt,
		realCompositeGridFunction & u,  
		realCompositeGridFunction & u0, 
		realCompositeGridFunction & u1, 
		realCompositeGridFunction & u2);

extern "C" 
{ 
void dlpoly_initialize_();  
void dlpoly_force_eval_(double*,double*);  
void dlpoly_terminate_();
void zigzag_(int*,float*,float*,float*);
} // prototype with C language linkage

void import_INI(aString,int,int,bool);
void import_INI(aString,int);
void import_CONFIG(doubleArray & position, doubleArray & velocity, int sim=0);
void rescale_position(doubleArray & position, doubleArray & velocity, doubleArray & position_ini, double Position_rescale, double Velocity_rescale);
void run_Sarw(sarw_data & dat, int sim=0);
void import_grid(const aString);
void export_FIELD_CONTROL(const char*, const char*, int NoOfWallAtoms);
void init_history(const char*, ofstream & history);
void export_history(ofstream & history, const doubleArray & position);
void export_psf(const char*);
void calculate_RDF(const doubleArray & position, doubleArray & RDF, int NoOfShells);
void ann(ANNkd_tree* kdTree, const doubleArray & position, const doubleArray & Dwall, doubleArray & Fwall, realArray & domain_min, realArray & domain_max, bool ProductionPhase, realCompositeGridFunction & D, realCompositeGridFunction & Dd);
//inline void forceEval(const doubleArray & p, doubleArray & f, doubleArray & fMS,doubleArray & fJEN,fe & fed); // make these inline to run faster
inline void forceEval(const doubleArray & p, doubleArray & f, fe & fed); // make these inline to run faster
inline void rndEval(doubleArray & Random, const doubleArray & Sigma, const double Sigma_reduced);
bool FileExists(string strFilename);
int getNumberOfPoints(CompositeGrid & gc);
void DrawProgressBar(int len, double percent);
void getDomainBounds(realArray & min, realArray & max, double L_rescale);
void getDomainBoundsIndicial(intArray & min, intArray & max);
void eval_Quadrupole(realCompositeGridFunction & E, realCompositeGridFunction & Q);
void CM_NS_Build(realCompositeGridFunction & coeff2, int matrix=1, Real dt=1);
void CM_NS_ApplyBCs(realCompositeGridFunction & coeff2);
void checkIfPenetration(const int & step, const doubleArray & position, realArray & min, realArray & max);
int checkIfInside(const int & step, const doubleArray & position, realArray & min, realArray & max);
void NDist(const float x, const float y, const float z, float Length, float H, float W, double & D, double & Dx, double & Dy, double & Dz, bool ProductionPhase);
int add_walls_to_CONFIG(const char * xyz_file, char * config_file, double Position_rescale);
int shift_position(doubleArray & position, double Position_rescale);
void write_position_to_CONFIG(doubleArray & position, char * config_file);
double variance( const doubleArray & arr, int no );
void FreeSpaceStokeslet(int sp1, int sp2, int sp3, realCompositeGridFunction & Omega_ob, double a, double kappa, double scale, int Stokeslet_Form);
void ComputeVD(doubleArray & position, doubleArray & force, realCompositeGridFunction & Omega, doubleArray & vd, Ogshow & show, bool Complete_Set, float H, float W);
void ComputeD(doubleArray position, realCompositeGridFunction & Omega, doubleArray & D, bool Complete_Set, int OI, realArray & timers, int Stokeslet_Form, double L_rescale, bool HI, bool WallHI, bool ElHI, bool ElWallHI, bool Full_Plane);
// ComputeOmega in Tracker.C:
void ComputeOmega(doubleArray & p, doubleArray & Omega, int Stokeslet_Form, bool HI, bool WHI);
double S(int k, int element);
double RPY(int k, int element, double & a);
double PDW(int k, double y0, int element, int yz);
double PD(int k, int element, int yz);
double SD(int k, int element, int yz);
// ComputeOmega in hi.C:
void ComputeOmega_hi(doubleArray p,doubleArray p0, doubleArray & G, bool HI, double omega_scale);
double S(doubleArray & r, int element);
double PDy(doubleArray & r, int element);
double PDz(doubleArray & r, int element);
double SDy(doubleArray & r, int element);
double SDz(doubleArray & r, int element);
void findWeightsForTriLinearInterpolation( realMappedGridFunction & vertex, intArray & FROM, doubleArray & to, doubleArray  & w);
//double triLinearInterpolation(doubleArray & weights, double & V000, double & V001, double & V010, double & V100, double & V110, double & V101, double & V011, double & V111);
double triLinearInterpolation(doubleArray & weights, double V0, double V1, double V2, double V3, double V4, double V5, double V6, double V7);
void chol(doubleArray & D, doubleArray & Sigma, int N, int & step, realArray & timers);
void ijk2ab(int & i, int & j, int & k, int & a, int & b);
void ijkl2ab(int & i, int & j, int & k, int & l, int & a, int & b);
void FreeSpaceStokeslet(doubleArray & D, int & i, int & j, doubleArray & r, double & a, double & kappa, double & scale, int Stokeslet_Form, realArray & timers);
void sourcePointLimits(const int & nx, const int & ny,const int & nz, int & sp1start, int & sp1stop, int & sp2start, int & sp2stop, int & sp2step, int & sp3start, int & sp3stop, int & sp3step, bool & Complete_Set, bool & Full_Plane, int & Walls);
void unitMatrix(doubleArray & D, const int & DSIZE);
void interpolateOmegaTriLinear(const realCompositeGridFunction & Omega, const int i, const int j, doubleArray & xpd, doubleArray & x0pd, doubleArray & D, realArray & timers, bool Full_Plane);
void interpolateOmegaNearestNeighbor(const realCompositeGridFunction & Omega, const int i, const int j, doubleArray & xpd, doubleArray & x0pd, doubleArray & D, realArray & timers, bool Full_Plane);
void printOmegas(int T, int sp1, int sp2, int sp3, realCompositeGridFunction & Om, int grid, int sp1o, int nx, int ny, int nz);
void checkOmegaSymmetry(int fp1,int fp2,int fp3, int sp1, int sp2, int sp3, realCompositeGridFunction & Omega);
void exportGridNodes(realMappedGridFunction & vertex,int x,int y,int z,ofstream & ofs);
void exportOmega(int fp1,int fp2,int fp3,int sp1,int sp2,int sp3,realCompositeGridFunction & Omega,ofstream & ofs_omega);
void differenceFormula(const intArray & r, const intArray & r0, intArray & DF, int Dim, int nx, int ny, int nz);
inline double distRij(int x,int y,int z,int x0,int y0,int z0) {return sqrt( (x-x0)*(x-x0) + (y-y0)*(y-y0) + (z-z0)*(z-z0) );}
void dDdrHdf2Txt(const intArray & R, const intArray & R0, const realCompositeGridFunction & dDdr, ofstream & ofs, int grid);
void Prod(const doubleArray & A, const doubleArray & x, doubleArray & y,realArray & timers);
void InvertAndTranspose(const doubleArray & B, int DN, realArray & timers);
void UpdateVelocity(doubleArray r, doubleArray & f, doubleArray & elf, doubleArray & n, doubleArray & v0, realCompositeGridFunction & Omega, realCompositeGridFunction & ElOmega, bool Complete_Set, int OI, realArray & timers, int Stokeslet_Form, bool HIs, double dt, double L_rescale, bool WallHI, bool ElHI, bool ElWallHI, bool DivD, doubleArray & v, doubleArray & v_aff, doubleArray & v1, doubleArray & v2, doubleArray & v3, bool Full_Plane);
void computeDivD(intArray R, intArray R0, const realCompositeGridFunction & Omega, realCompositeGridFunction & dDdr, int Dim, int grid, bool Debug, int nx, int ny, int nz);
void export_textdata(ofstream & textdata, const doubleArray & position, const doubleArray & com, const doubleArray & v0,const doubleArray & v1,const doubleArray & v2,const doubleArray & v3);
int symmetryTransformIndicial(const int & sp1o, const int & sp1, const int & sp2, const int & sp3, const int & i1, const int & i2, const int & i3, int & x0, int & y0, int & z0, int & x, int & y, int & z, const int & row, const int & col, const int h, const int w, bool Complete_Set, bool Full_Plane);
int anchorDumbell(const doubleArray & position, int i, int j, doubleArray & xpd, doubleArray & x0pd, bool Full_Plane, const Real & PDlength, const Real & SPplane);
void rotateDumbell(int & y0, int & y, int & z0, int & z, int r, int c,  Real h, Real w);
double Om(const int m, const int n, const int r, const int c, const intArray & NN0, const intArray & NN, const realCompositeGridFunction & Omega, bool Full_Plane);
void NNSearchOnRegularGrid(const doubleArray & xpd, const realArray & del, intArray & NN, const int & NNN);
void getDomainGridSpacings(realArray & domain_max, realArray & domain_del);
void displayDebugInfo(const int i, const int j, const int r, const int c, const int NNN, const doubleArray & x0pd, const doubleArray & xpd, const intArray & NN, const intArray & NN0, const doubleArray & weights, const doubleArray & weights0, const doubleArray & Gss, const realCompositeGridFunction & Omega, bool Full_Plane);
void displayDebugInfoNN(const int i, const int j, const int r, const int c, const int NNN, const doubleArray & x0pd, const doubleArray & xpd, const intArray & NN, const intArray & NN0, const realCompositeGridFunction & Omega, bool Full_Plane);
double compute2DPoiseuilleFlow(const double y, const double z, const int N, const double vmax);


//=========================================================================================
//  Namespaces for sharing variables among functions
//=========================================================================================
namespace Data_ini {
bool
  Momentum_transfer,
  False_transients,
  BD,
  P_Oseen,
  Tracers,
  Electrophoretic,
  Dielectrophoretic,
  Molecular_dynamics,
  Electroosmotic, 
  Shear_driven,
  Pressure_driven,
  Electric_field,
  Plot_intermediate,
  Plot_final,
  Electrostatics,
  Export_vmd,
  Com,
  Msrgyr,
  Mseed,
  Stretch,
  Rmsd,
  MsdVaf,
  Cov,
  Rdf;

int
  Vdw,
  Wall,
  Debug,
  SpringLaw,
  LengthScale,
  TimeScale,
  ForceScale,
  OrderOfAccuracy, 
  NoOfParticles,
  NoOfTimeStepsT,
  NoOfTimeSteps,
  NoOfSims,
  N1, 
  N2, 
  N3,
  Mode,
  StepMax, 
  ErrorFreq,
  MTFreq,
  Direction,
  NoOfShells,
  ExportFreq;

long Seed;

float 
  BOND,
  ALPHA,
  THETA;

double 
  ks,
  bk,
  q0,
  vdw_epsilon,
  vdw_sigma,
  dtT, 
  dt,
  dt_d,
  dt_u, 
  dt_v, 
  eps_d,
  eps_u,
  eps_v,
  debye,
  epsilon,
  eta,
  rho,
  temp,
  dep_corr,
  height,
  i_wall, 
  i_wall2,
  a_inflow1, 
  a_inflow2, 
  a_outflow1, 
  a_outflow2,
  charge_p,
  rad_p,
  mass_p,
  lambda,
  shear_rate,
  pressure,
  Aw,
  com_ref_x,
  com_ref_y,
  com_ref_z,
  equil_frac,
  nof,
  friction_corr, 
  v_ev;

aString 
  nameOfOGFile,
  outputSuffix;
}

namespace Main_misc { 
  // Misc vars
  const int 
    Singleton=1, Null=0, c0=0, c1=1, c2=2, c3=3,
    eq1=0, eq2=1, eq3=2, eq4=3,  // equation numbers
    ic=0, ac=1,    // component numbers (intrinsic, applied el. potentials)
    pc=0, uc=1, vc=2, wc=3;      // component numbers
  int  i1, i2, i3, Dim, Dim0, Dim1, Dim2, d1, d2, d3,
    particle, particle2, dim, dim2, step, ft_step, sim,
    origin, delay, n, i, j, k, side, axis, grid;
  double SUM, error_u, error_d, error_v, error_p, C_wall, A_wall, B_wall, delta_w; 
  char buffer[80];
  double L, Tau, u_0, f_0, phi_i_0, phi_a_0, E_0, p_0, zeta_0, mass_0; // Scales for non-dimensionalization
  Real spacing, damping;
  Real t,
    time0=0,
    time00=0,
    time_START=0,
    time_INI=0,
    time_EIK=0,
    time_PB=0,
    time_W=0,
    time_NS=0,
    time_TD=0,
    time_PD=0,
    time_REALINI=0,
    time_REALFIN=0,
    time_HISTEXPORT=0,
    time_DLPOLY_START=0,
    time_DLPOLY=0,
    time_INTEG=0,
    time_RHSUPDATE=0,
    time_NSUPDATE=0,
    time_MTRANSFER=0, 
    time_PDINI=0,
    time_PPEVAL=0,
    time_PPEXPORT=0,
    time_COM=0,
    time_MSRGYR=0,
    time_MSEED=0,
    time_STRETCH=0,
    time_MSD=0; // declare & initialize timers
  Real timeForBoundaryConditions=0.,
    timeForSolveInterpolate=0.,
    timeForDudt=0.;
  Range all; // null range (entire range)
  Range R0(0,0), R1(1,1), R2(2,2), R1D, RDim(0,Dim); // component ranges (R0 also use a dummy dimension)
  Index I1, I2, I3, Ib1, Ib2, Ib3, Ig1, Ig2, Ig3, Ig21, Ig22, Ig23, Ig_1, Ig_2, Ig_3;
  deque< realCompositeGridFunction > Deque; // create a double-ended queue of CGFs
  // Make some shorter names for BCs for readability
  BCTypes::
    BCNames dirichlet     = BCTypes::dirichlet,
    neumann               = BCTypes::neumann,
    normalComponent       = BCTypes::normalComponent,
    extrapolate           = BCTypes::extrapolate,
    generalizedDivergence = BCTypes::generalizedDivergence,
    allBoundaries         = BCTypes::allBoundaries;
}

  namespace Adams_Bashforth {
 double 
   b3=55./24., // b constants
   b2=-59./24., 
   b1=37./24., 
   b0=-3./8.;
  }

 namespace PP {
   double Diff;
   int prodstep, NoOfProdSteps, NoOfEquilSteps;
   int NT0, NorgT, iorg, dorg, Norg;
   int 
     row_rg00=0, 
     row_rg01=0, 
     row_rg02=0, 
     row_rg10=0, 
     row_rg11=0, 
     row_rg12=0, 
     row_rg20=0, 
     row_rg21=0, 
     row_rg22=0, 

     row_msrgyr=0, 
     row_mseed=0, 
     row_stretch=0, 
     row_msd=0,
     row_vaf=0,
     row_com=0,
     row_diff_msd=0,
     row_diff_vaf=0,
     row_vcom_avg=0,
     row_transloc=0;
   intArray PPDrows(19), PPSrows(19), PPSIMrows(19);
 }

 namespace Symmetry_masks { // mask arrays for symmetryTransform()
   intArray 
     sgn(3,3),
     sgnx(3,3),sgny(3,3),sgnz(3,3), // these will remain constant (reflect sign changes associated with symmetry transforms of Omega)
     sgnX(3,3),sgnY(3,3),sgnZ(3,3); // these will vary
 }

namespace Domain_bounds { // domain bounds (real and indicial)
  realArray domain_min(3),domain_max(3), domain_del(3);
  intArray domain_min_ind(3),domain_max_ind(3);
  realMappedGridFunction cg0vertex;
 }

namespace NNSearch { // NNSearchOnRegularGrid()
  intArray NNM(8,3); // nearest neighbor mask (8 neigbors in 3D)
 }

namespace Cmdline_args {
  int NNS, DIF;
 }

namespace ComputeD_data {
  int ElFieldDir;
  double a_hat, kappa_hat, mu0_hat, PeL, omega_scale, elomega_scale;
  Real PDlength, SPplane;
}

namespace ComputeOmega_data {
  doubleArray r(5,3), rm(5), rm2(5);
}
#endif
